/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.javafx.control.treetableview;

import static javafx.embed.swt.SWTFXUtils.toFXImage;
import static org.fortiss.tooling.kernel.ui.util.KernelUIUtils.getName;

import java.util.ArrayList;
import java.util.List;

import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.common.ui.javafx.AF3FXViewPart;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeContentProviderBase;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeItemBase;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeUIProviderBase;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeViewer;
import org.fortiss.tooling.kernel.model.INamedElement;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

import javafx.scene.Node;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TreeView;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;

/**
 * This class is a support fragment for creating tree views in JavaFX Controls. It provides the view
 * counter part of a {@link DynamicTreeContentProviderBase}. The content provider is responsible
 * for selecting elements from a base model and ordering them according to a custom defined
 * structure.
 * These two fragments have to be combined in a JavaFX controller inheriting from the base class
 * {@link AF3FXViewPart}.
 * <p>
 * TreeViewer that constructs a DynamicTreeViewer whose UI provider uses the methods and kernel
 * services for {@link INamedElement}s.
 * The Viewer uses a composition principle where a client must provide a {@link TreeView} (JavaFX),
 * a root element (AF3), and a content provider that selects the elements to be displayed.
 * 
 * @author diewald
 */
public class ModelElementTreeViewer<T extends INamedElement> {

	/** {@link DynamicTreeUIProviderBase} for named (and commented) {@link IModelElement}s . */
	protected class ModelElementUIProvider extends DynamicTreeUIProviderBase<T> {
		/** {@inheritDoc} */
		@Override
		public String getLabel(T element) {
			return getName(element);
		}

		/** {@inheritDoc} */
		@Override
		public Node getIconNode(T element) {
			org.eclipse.swt.graphics.Image icon =
					IModelElementHandlerService.getInstance().getIcon(element);
			if(icon != null) {
				Image fxImage = toFXImage(icon.getImageData(), null);
				return new ImageView(fxImage);
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public ContextMenu createContextMenu(T element) {
			ContextMenu menu = new ContextMenu();
			contextMenuEntries.forEach(entry -> menu.getItems().add(entry));
			return menu;
		}
	}

	/** References the constructed {@link DynamicTreeViewer} "controller". */
	protected DynamicTreeViewer<T> dynTreeViewer;

	/**
	 * List of context menu entries that supplied by clients (see
	 * {@link #addContextMenuEntry(MenuItem)}).
	 */
	protected List<MenuItem> contextMenuEntries = new ArrayList<>();

	/** Constructor. */
	public ModelElementTreeViewer(TreeView<T> treeView, T modelRoot,
			DynamicTreeContentProviderBase<T> contentProvider) {
		DynamicTreeUIProviderBase<T> uiProvider = createUIProvider();
		dynTreeViewer =
				new DynamicTreeViewer<T>(treeView, modelRoot, true, 0, contentProvider, uiProvider);
	}

	/**
	 * Creates a default UI provider for {@link DynamicTreeViewer}s that present
	 * {@link IHierarchicElement}s. It also uses the {@link IModelElementHandlerService} for a nicer
	 * visual appearance.
	 * 
	 * @return The constructed UI Provider.
	 */
	protected DynamicTreeUIProviderBase<T> createUIProvider() {
		return new ModelElementUIProvider();
	}

	/**
	 * Method to define context {@link MenuItem}s for items of the underlying content. The given
	 * function may use the selected element (given as a parameter to the function) to define
	 * dynamic behavior.
	 *
	 * @param menuItem
	 *            {@link MenuItem} and its logic to be added to the context menu.
	 */
	public void addContextMenuEntry(MenuItem menuItem) {
		contextMenuEntries.add(menuItem);
	}

	/** Update the internal viewer. */
	public void update() {
		dynTreeViewer.update();
	}

	/** Expands the tree down to the given value. */
	public void expandItem(T value) {
		DynamicTreeItemBase<T> item = dynTreeViewer.findItem(value);
		dynTreeViewer.expandItem(item);
	}
}
